@@ -1,5 +1,5 @@
 package XML::TMX::CWB;
-$XML::TMX::CWB::VERSION = '0.07';
+$XML::TMX::CWB::VERSION = '0.08';
 use warnings;
 use strict;
 use Lingua::PT::PLNbase;
@@ -84,6 +84,18 @@ Imports a TMX file (just two languages) to a parallel corpus on CWB.
 
 =cut
 
+sub _get_header_prop_list {
+    my ($reader, $prop) = @_;
+
+    return undef unless exists $reader->{header}{-prop}{$prop};
+
+    my $value = (ref($reader->{header}{-prop}{$prop}) eq "ARRAY")
+              ? join(",",@{$reader->{header}{-prop}{$prop}})
+              : $reader->{header}{-prop}{$prop};
+
+    return [ split /\s*,\s*/ => $value ];
+}
+
 
 sub _RUN {
     my $command = shift;
@@ -105,37 +117,32 @@ sub toCWB {
     # Create reader object
     my $reader = XML::TMX::Reader->new($tmx);
 
-    my $tagged = exists($reader->{header}{-prop}{tagged})
-      && (
-          (
-           ref($reader->{header}{-prop}{tagged}) eq "ARRAY"
-           &&
-           ($reader->{header}{-prop}{tagged}[0] eq "true")
-          ) || (
-                $reader->{header}{-prop}{tagged} eq "true"
-               )
-         );
+    my %tagged_languages = ();
+    for my $language (@{ _get_header_prop_list($reader, "pos-tagged") }) {
+        $tagged_languages{$language}++;
+    }  
 
-    if ($tagged && ($ops{tok} || $ops{tokenize_source} || $ops{tokenize_target})) {
-        warn "Can't tokenize a tagged file. Ignoring tokenization request.";
-        delete $ops{$_} for (qw'tok tokenize_source tokenize_target');
-    }
+    my $has_tagged_languages = scalar(keys %tagged_languages);
 
-    if ($tagged) {
-        die "Can't guess fields for tagged tmx" unless exists $reader->{header}{-prop}{fields};
+    my $tag_data = undef;
+    my $s_attributes = ['s'];
+    my $pos_fields = [qw'word lemma pos'];
 
-        my $sattr =
-          exists($reader->{header}{-prop}{'s-attribute'})
-            ?
-              $reader->{header}{-prop}{'s-attribute'}
-                :
-                  []
-                    ;
+    if ($has_tagged_languages) {
 
-        $tagged = {
-                   fields => [split /,/ => $reader->{header}{-prop}{fields}[0]],
-                   sattr  => $sattr,
-                  };
+        if ($ops{tok} || $ops{tokenize_source} || $ops{tokenize_target}) {
+            warn "Can't tokenize tagged languages. Ignoring tagging request for ",
+                join(", ", keys %tagged_languages);
+        }
+
+        $s_attributes ||= _get_header_prop_list('pos-s-attributes');
+        $pos_fields   ||= _get_header_prop_list('pos-fields');
+
+        $tag_data = {
+            languages => \%tagged_languages,
+            'pos-s-attributes' => $s_attributes,
+            'pos-fields' => $pos_fields
+        };
     }
 
     my ($source, $target);
@@ -158,25 +165,25 @@ sub toCWB {
     chomp( $registry = `cwb-config -r` ) unless $registry;
     die "Could not detect a suitable CWB registry folder.\n" unless $registry && -d $registry;
 
+    # detect corpus name
     my $cname = $ops{corpus_name} || $tmx;
-
     $cname =~ s/[.-]/_/g;
 
     if ($ops{mono}) {
         _mtmx2cqpfiles($reader, $cname, $source,
                        $ops{tok} || 0,
                        $ops{verbose} || 0,
-                       $tagged
+                       $tag_data
                       );
-        _mencode($cname, $corpora, $registry, $source, $tagged);
+        _mencode($cname, $corpora, $registry, $source, $tag_data);
     } else {
         _tmx2cqpfiles($reader, $cname, $source, $target,
                       $ops{tokenize_source} || 0,
                       $ops{tokenize_target} || 0,
                       $ops{verbose} || 0,
-                      $tagged
+                      $tag_data
                      );
-        _encode($cname, $corpora, $registry, $source, $target, $tagged);
+        _encode($cname, $corpora, $registry, $source, $target, $tag_data);
         unlink "target.cqp";
         unlink "align.txt";
     }
@@ -186,29 +193,33 @@ sub toCWB {
 
 sub _encode {
     my ($cname, $corpora, $registry, $l1, $l2, $tagged) = @_;
-    my ($name, $folder, $reg);
 
-    my ($posatt, $sattr) = ("", "");
-    if ($tagged) {
-       shift @{$tagged->{fields}};
-       $posatt = join(" ", map { "-P $_" } @{$tagged->{fields}});
-       $sattr  = join(" ", map { "-S $_" } @{$tagged->{sattr}});
+    my @languages = ($l1, $l2);
+    my @files = (qw'source target');
+    
+    for my $i (0, 1) {
+        my ($name, $folder, $reg);
+        my ($posatt, $sattr) = ("", "");
+
+        my $l = $languages[$i];
+        my $f = $files[$i];
+
+        if ($tagged && exists($tagged->{languages}{$l})) {
+           shift @{$tagged->{'pos-fields'}};
+           $posatt = join(" ", map { "-P $_" } @{$tagged->{'pos-fields'}});
+           $sattr  = join(" ", map { "-S $_" } @{$tagged->{'pos-s-attributes'}});
+        }        
+    
+        $name  = lc("${cname}_$l");
+        $folder = catfile($corpora,  $name);
+        $reg    = catfile($registry, $name);
+
+        mkdir $folder;
+
+        _RUN("cwb-encode -c utf8 -d $folder -f $f.cqp -R $reg -S tu+id $sattr $posatt");
+        _RUN("cwb-make -r $registry -v " . uc($name));
     }
 
-    $name = lc("${cname}_$l1");
-    $folder = catfile($corpora,  $name);
-    $reg    = catfile($registry, $name);
-    mkdir $folder;
-    _RUN("cwb-encode -c utf8 -d $folder -f source.cqp -R $reg -S tu+id $sattr $posatt");
-    _RUN("cwb-make -r $registry -v " . uc($name));
-
-    $name = lc("${cname}_$l2");
-    $folder = catfile($corpora,  $name);
-    $reg    = catfile($registry, $name);
-    mkdir $folder;
-    _RUN("cwb-encode -c utf8 -d $folder -f target.cqp -R $reg -S tu+id $sattr $posatt");
-    _RUN("cwb-make -r $registry -v " . uc($name));
-
     _RUN("cwb-align-import -r $registry -v align.txt");
     _RUN("cwb-align-import -r $registry -v -inverse align.txt");
 }
@@ -221,10 +232,10 @@ sub _mencode {
     my $reg    = catfile($registry, $name);
 
     my ($posatt, $sattr) = ("", "");
-    if ($tagged) {
-       shift @{$tagged->{fields}};
-       $posatt = join(" ", map { "-P $_" } @{$tagged->{fields}});
-       $sattr  = join(" ", map { "-S $_" } @{$tagged->{sattr}});
+    if ($tagged && $tagged->{languages}{$l1}) {
+       shift @{$tagged->{'pos-fields'}};
+       $posatt = join(" ", map { "-P $_" } @{$tagged->{'pos-fields'}});
+       $sattr  = join(" ", map { "-S $_" } @{$tagged->{'pos-s-attributes'}});
     }
 
     mkdir $folder;
@@ -248,27 +259,32 @@ sub _tmx2cqpfiles {
         return unless exists $langs->{$l1} && exists $langs->{$l2};
 
         my (@S, @T);
-        if (not $tagged) {
-            for ($langs->{$l1}{-seg}, $langs->{$l2}{-seg}) {
+        
+        # Language 1
+        if ($tagged && exists($tagged->{languages}{$l1})) {
+             @S = split /\n/, $langs->{$l1}{-seg};
+        }
+        else {
+            for ($langs->{$l1}{-seg}) {
+                s/&/&amp;/g;
+                s/</&lt;/g;
+                s/</&lt;/g;
+            }
+            @S = $t1 ? tokenize($langs->{$l1}{-seg}) : split /\s+/, $langs->{$l1}{-seg};
+        }
+
+        # Language 2
+        if ($tagged && exists($tagged->{languages}{$l2})) {
+            @T = split /\n/, $langs->{$l2}{-seg};
+        }
+        else {
+            for ($langs->{$l2}{-seg}) {
                 s/&/&amp;/g;
                 s/</&lt;/g;
                 s/</&lt;/g;
             }
 
-            @S = $t1 ? tokenize($langs->{$l1}{-seg}) : split /\s+/, $langs->{$l1}{-seg};
             @T = $t2 ? tokenize($langs->{$l2}{-seg}) : split /\s+/, $langs->{$l2}{-seg};
-
-        } else {
-            @S = split /\n/, $langs->{$l1}{-seg};
-            @T = split /\n/, $langs->{$l2}{-seg};
-
-            for (@S, @T) {
-                if (/\t/) {
-                    s/&/&amp;/g;
-                    s/</&lt;/g;
-                    s/>/&gt;/g;
-                }
-            }
         }
 
         print STDERR "\rProcessing... $i translation units" if $v && $i%1000==0;
@@ -298,7 +314,9 @@ sub _mtmx2cqpfiles {
         return unless exists $langs->{$l1};
 
         my (@S);
-        if (not $tagged) {
+        if ($tagged && exists($tagged->{languages}{$l1})) {
+            @S = split /\n/, $langs->{$l1}{-seg};
+        } else {
             for ($langs->{$l1}{-seg}) {
                 s/&/&amp;/g;
                 s/</&lt;/g;
@@ -306,18 +324,7 @@ sub _mtmx2cqpfiles {
             }
 
             @S = $t1 ? tokenize($langs->{$l1}{-seg}) : split /\s+/, $langs->{$l1}{-seg};
-        } else {
-            @S = split /\n/, $langs->{$l1}{-seg};
-
-            for (@S) {
-                if (/\t/) {
-                    s/&/&amp;/g;
-                    s/</&lt;/g;
-                    s/>/&gt;/g;
-                }
-            }
         }
-
         print STDERR "\rProcessing... $i translation units" if $v && $i%1000==0;
 
         print F1 "<tu id='$i'>\n", join("\n", @S), "\n</tu>\n";
